#if defined(WIN32)

   /*
   ** Windows specific includes.
   */
   #include<windows.h>

#elif defined(N_PLAT_NLM)

   /*
   ** NetWare NLM specific includes.
   */

   #include <nwthread.h>
   #include <string.h>

#define printf ConsolePrintf

#elif defined(UNIX)

   /*
   ** Unix specific includes.
   */

   #include <ldap.h>
   #include <lber.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <ntypes.h>
#include <nmaserr.h>
#include <nmasext.h>
#include <ldap_ssl.h>

#define MAX_DN_CHARS                    256
#define MAX_TREE_NAME_CHARS             32

/* ******************************************************************* */
void findAnObject(LDAP *ld, char *queryID)
{
   const char *apszAttributes[] =
   {
         "mail",
         "cn",
         NULL
   };

   int err;
    char searchBase[] = ""; // serach the entire tree
    struct timeval timeOut;
    LDAPMessage *searchResult;

    timeOut.tv_sec    = 10L;
    timeOut.tv_usec   = 0L;

    /* Search the directory */
    err = ldap_search_ext_s(
                    ld,                    /* LDAP session handle */
                    searchBase,            /* container to search */
                    LDAP_SCOPE_SUBTREE,    /* search scope */
                    queryID,            /* search filter */
                    (char **)apszAttributes,        /* return specified attributes */
                    0,                     /* return attributes and values */
                    NULL,                  /* server controls */
                    NULL,                  /* client controls */
                    &timeOut,              /* time out */
                    LDAP_NO_LIMIT,         /* no size limit */
                    &searchResult );       /* returned results */

    ldap_msgfree( searchResult );
}

/* ******************************************************************* */
int main(int argc, char **argv)
{
   int err = 0;

   LDAP *ld = NULL;
   int LDAPVersion = LDAP_VERSION3;
   int resetCur = 0;
   char polcDN[MAX_DN_CHARS +1];
   size_t polcDNLen = sizeof(polcDN);
   char subjName[MAX_DN_CHARS +1];
   char subjPwd[512];
   char userName[MAX_DN_CHARS +1];
   BerValue cred = {0};
   char userPwd[512];
   char chgPwd[]="This is the changed password";
   char newPwd[] ="THE NEW PASSWORD";
   char curPwd[512];
   char getPwd[512];
   size_t getPwdLen = sizeof(getPwd);
   nuint32 mid = 0;
   nuint32 midLen = sizeof(mid);
   char tag[] = "BIG BUFFER";

   char bigBuf[60000];
   char getBigBuf[60000];
   size_t bigBufLen = sizeof(bigBuf);
   size_t getBigBufLen = sizeof(getBigBuf);
   char *ipAddr, *derFile;


   if(argc < 7 )
   {
      printf("sample usage: <ldap ip addr> <der file> <subjDN> <subjPwd> <objDN> <objPwd>\n");
      return -1;
   }
   ipAddr = argv[1];

   derFile = argv[2];

   strcpy( subjName, argv[3]);

   strcpy( userName, argv[5]);

   strcpy( userPwd, argv[6]);

   // establish LDAP connection
   ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &LDAPVersion);

     // initialize the ssl library
   err = ldapssl_client_init(derFile, NULL );
   if (err != LDAP_SUCCESS)
   {
      printf ("ldapssl_client_init failed %d\n", err);
      return err;
   }

   // Initialize the ldap connection for ssl
   if (( ld = ldapssl_init( ipAddr, 636, 1 )) == NULL)
   {
      err = NMAS_E_SERVER_NOT_FOUND;
      printf ("(LSMLDAP)- ldapssl_init failed\n");
   }

   if (!err)
   {
      // change password -- anonymous connection
      err = nmasldap_change_password( ld, userName, userPwd, chgPwd);
      printf("ERROR %d nmasldap_change_password\n", err);

      // get password policy DN
      polcDNLen = sizeof(polcDN);
      err = nmasldap_get_password_policy_dn(ld, userName, &polcDNLen, polcDN);
      if (!err)
      {
         printf("PUBLIC nmasldap_get_password_policy_dn %s\n", polcDN);
      }
      else
      {
         printf("ERROR %d PUBLIC nmasldap_get_password_policy_dn\n", err);
      }

      // Perform Simple BIND
      cred.bv_val = (char *)malloc(strlen(argv[4]));
      strcpy(cred.bv_val, argv[4]);
      cred.bv_len = strlen(argv[4]);
      err = ldap_sasl_bind_s(ld,
         subjName,
         LDAP_SASL_SIMPLE,
         &cred,
         NULL,
         NULL,
         NULL);
      free(cred.bv_val);
      if (err != LDAP_SUCCESS)
      {
         printf("ERROR %d ldap_simple_bind_s\n", err);
      }
      else
      {

           nuint32 pwdStatus=0xffffffff;
      nuint32 simplePwdStatus=0xffffffff;

         // get password status
            err = nmasldap_get_password_status(ld, userName, &pwdStatus, &simplePwdStatus);
         if (!err)
         {
            printf("Universal Password status: 0x%x\n", pwdStatus);
            printf("Simple Password status: 0x%x\n", simplePwdStatus);
         }
         else
         {
            printf("ERROR %d nmasldap_get_password_status\n", err);
         }

         // get password policy DN
         polcDNLen = sizeof(polcDN);
         err = nmasldap_get_password_policy_dn(ld, userName, &polcDNLen, polcDN);
         if (!err)
         {
            printf("nmasldap_get_password_policy_dn %s\n", polcDN);
         }
         else
         {
            printf("ERROR %d nmasldap_get_password_policy_dn\n", err);
         }

         /* **************************************************************** */
         // Universal Password
         /* **************************************************************** */

         // get current password
         getPwdLen = sizeof(getPwd);
         err = nmasldap_get_password( ld, userName, &getPwdLen, getPwd);
         printf("ERROR %d nmasldap_get_password\n", err);
         if (!err)
         {
            resetCur = 1;

            if (strcmp(chgPwd, getPwd) == 0)
            {
               printf("change and get passwords are the same\n");
            }
            else
            {
               printf("change and get passwords are NOT the same\n");
            }
         }


         // check current password
         err = nmasldap_policy_check_current_password(ld, userName);
         printf("ERROR %d nmasldap_policy_check_password\n", err);

         // check password
         err = nmasldap_policy_check_password(ld, userName, newPwd);
         printf("ERROR %d nmasldap_policy_check_password\n", err);

         // change password
         err = nmasldap_change_password( ld, userName, getPwd, chgPwd);
         printf("ERROR %d nmasldap_change_password\n", err);
         getPwdLen = sizeof(getPwd);
         err = nmasldap_get_password( ld, userName, &getPwdLen, getPwd);
         printf("ERROR %d nmasldap_get_password\n", err);

         if (!err)
         {
            if (strcmp(chgPwd, getPwd) == 0)
            {
               printf("change and get passwords are the same\n");
            }
            else
            {
               printf("change and get passwords are NOT the same\n");
            }
         }

         // set password
         err = nmasldap_set_password( ld, userName, newPwd);
         printf("ERROR %d nmasldap_set_password\n", err);
         getPwdLen = sizeof(getPwd);
         err = nmasldap_get_password( ld, userName, &getPwdLen, getPwd);
         printf("ERROR %d nmasldap_get_password\n", err);

         if (!err)
         {
            if (strcmp(newPwd, getPwd) == 0)
            {
               printf("set and get passwords are the same\n");
            }
            else
            {
               printf("set and get passwords are NOT the same\n");
            }
         }

         err = nmasldap_delete_password( ld, userName);
         printf("ERROR %d nmasldap_delete_password\n", err);

         err = nmasldap_set_password( ld, userName, "a"); // set to short password
         printf("ERROR %d nmasldap_set_password(a)\n", err);

         err = nmasldap_set_password( ld, userName, "novell"); // set to novell password
         printf("ERROR %d nmasldap_set_password(novell)\n", err);

         err = nmasldap_set_password( ld, userName, "5novell"); // set to first numeric character
         printf("ERROR %d nmasldap_set_password(5novell)\n", err);

         err = nmasldap_set_password( ld, userName, "novell5"); // set to last numeric character
         printf("ERROR %d nmasldap_set_password(novell5)\n", err);

         if (resetCur)
         {
            err = nmasldap_set_password( ld, userName, userPwd); // set password back to original
            printf("ERROR %d nmasldap_set_password\n", err);
         }
         else
         {
            err = nmasldap_set_password( ld, userName, newPwd); // set password back to new
            printf("ERROR %d nmasldap_set_password\n", err);
         }

         /* **************************************************************** */
         // Simple Password

         /* **************************************************************** */

         getPwdLen = sizeof(curPwd);
         err = nmasldap_get_simple_pwd( ld, userName, getPwdLen, curPwd);
         printf("ERROR %d nmasldap_get_simple_pwd\n", err);

         err = nmasldap_put_simple_pwd( ld, userName, newPwd);
         printf("ERROR %d nmasldap_put_simple_pwd\n", err);

         getPwdLen = sizeof(getPwd);
         err = nmasldap_get_simple_pwd( ld, userName, getPwdLen, getPwd);
         printf("ERROR %d nmasldap_get_simple_pwd\n", err);

         if (!err)
         {
            if (strcmp(newPwd, getPwd) == 0)
            {
               printf("set and get simple passwords are the same\n");
            }
            else
            {
               printf("set and get simple passwords are NOT the same\n");
            }
         }

         err = nmasldap_delete_simple_pwd( ld, userName);
         printf("ERROR %d nmasldap_delete_simple_pwd\n", err);


         /* **************************************************************** */
         // Login Configuration
         /* **************************************************************** */

         memset (bigBuf, '$', bigBufLen);
         err = nmasldap_put_login_config( ld, userName, midLen, &mid, tag, bigBufLen, bigBuf);
         printf("ERROR %d nmasldap_put_login_config (bigBuf)\n", err);

         err = nmasldap_get_login_config( ld, userName, midLen, &mid, tag, &getBigBufLen, getBigBuf);
         printf("ERROR %d nmasldap_get_login_config (getBigBuf)\n", err);
         if (!err)
         {
            if (bigBufLen != getBigBufLen)
            {
               printf("put %d and get %d length are different \n", bigBufLen, getBigBufLen);
            }
            else if (memcmp(bigBuf, getBigBuf, bigBufLen) != 0)
            {
               printf("put and get data are different \n");
            }
            else
            {
               printf("put and get data are the same \n");
            }
         }

         err = nmasldap_delete_login_config( ld, userName, midLen, &mid, tag);
         printf("ERROR %d nmasldap_delete_login_config\n", err);

         /* **************************************************************** */
         // Login Secret
         /* **************************************************************** */

         err = nmasldap_put_login_secret( ld, userName, midLen, &mid, tag, bigBufLen, bigBuf);
         printf("ERROR %d nmasldap_put_login_secret\n", err);

         err = nmasldap_delete_login_secret( ld, userName, midLen, &mid, tag);
         printf("ERROR %d nmasldap_delete_login_secret\n", err);
      }

      // unbind
      ldap_unbind_ext_s(ld, NULL, NULL);


      ldapssl_client_deinit();
   }

   return err;
}
