/*****************************************************************************

  %name: nmasclnt.h %
  %version: 10 %
  %version: 10 %
  $Copyright:

  Copyright (c) 1999 - 2007 Novell, Inc.  All Rights Reserved.                      

  THIS WORK IS  SUBJECT  TO  U.S.  AND  INTERNATIONAL  COPYRIGHT  LAWS  AND
  TREATIES.   NO  PART  OF  THIS  WORK MAY BE  USED,  PRACTICED,  PERFORMED
  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,  ABRIDGED, CONDENSED,
  EXPANDED,  COLLECTED,  COMPILED,  LINKED,  RECAST, TRANSFORMED OR ADAPTED
  WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION
  OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
  CRIMINAL AND CIVIL LIABILITY.$

 ****************************************************************************/

#ifndef _NMASCLNT_H
#define _NMASCLNT_H

#include <nmasclientapi.h>
#include <nmasaid.h>
#include <nmasmaf.h>
#include <nmasconf.h>
#include <nmaserr.h>

   /* NMAS requires unicode characters to be 16 bits on all platforms.  
    * The "unicode" type is declared below as a 16 bit value.
    * Define UNICODE_TYPE_DEFINED to bypass the typedef below if "uniocde" is
    * previously defined.
   */
#ifndef UNICODE_TYPE_DEFINED
#define UNICODE_TYPE_DEFINED
typedef unsigned short  unicode;
typedef unicode * punicode;
#endif


/*************************************************************************
   NMAS client login functionality 
*************************************************************************/

   /* NMAS_ClientLogin option flags
   */
#define NMAS_OPT_RESERVED1                0x00000001
#define NMAS_OPT_RESERVED2                0x00000002
#define NMAS_OPT_RESERVED4                0x00000004
#define NMAS_OPT_NO_CRYPTO                0x00000008
#define NMAS_OPT_RESERVED10               0x00000010
#define NMAS_OPT_NO_PROMPT_FOR_PWD        0x00000020
#define NMAS_OPT_LOGIN_ONLY               0x00000040
#define NMAS_OPT_NO_METHOD_UI             0x00000080
#define NMAS_OPT_VERIFY                   0x00000100


   /* NMAS_ClientLogin LoginInfo struct DN_Type flags
   */
#define NMAS_DN_TYPE_NDS_DOT              1  
#define NMAS_DN_TYPE_LDAP                 2  


   /* NMAS_ClientLoginEx login info data types
   */
#define NMAS_LOGIN_INFO_UNI               1   /* NMAS_uniLoginInfo   */
#define NMAS_LOGIN_INFO_UTF8              2   /* NMAS_utf8LoginInfo  */
#define NMAS_LOGIN_INFO_WCHAR             3   /* NMAS_wcharLoginInfo */      


   /* The Tree name in the structures below is optional and is only
    * applicable if the dnType is NMAS_DN_TYPE_NDS_DOT
    */

   /*  "unicode" strings - Novell specific 
   */
typedef struct _NMAS_uniLoginInfo
{
   unicode  *uniTree;         /* NDS tree name, optional */
   unicode  *uniUserDN;
   int       dnType;
   unicode  *uniPassword;
   unicode  *uniSequence;
   unicode  *uniClearance;
   void     *uiHandle;
   int       uiTimeout;
} NMAS_uniLoginInfo, *pNMAS_uniLoginInfo;

   /* utf8 strings
   */
typedef struct _NMAS_utf8LoginInfo
{
   char     *utf8Tree;        /* NDS tree name, optional */
   char     *utf8UserDN;
   int       dnType;
   char     *utf8Password;
   char     *utf8Sequence;
   char     *utf8Clearance;
   void     *uiHandle;
   int       uiTimeout;
} NMAS_utf8LoginInfo, *pNMAS_utf8LoginInfo;


   /* wchar_t strings - platform specific 
   */
typedef struct _NMAS_wcharLoginInfo
{
   wchar_t    *wTree;         /* NDS tree name, optional */
   wchar_t    *wUserDN;
   int         dnType;
   wchar_t    *wPassword;
   wchar_t    *wSequence;
   wchar_t    *wClearance;
   void       *uiHandle;
   int         uiTimeout;
} NMAS_wcharLoginInfo, *pNMAS_wcharLoginInfo;


   /* transport callback - called by client to send data
   */
typedef int (NMAS_CALLBACK *pFunc_NMAS_Transport)(
   void   *arg,               
   void   *instructionBuf, 
   size_t  instructionLen,
   void   *resultBuf, 
   size_t  resultLen );

   /* atEnd processing callback - called by client before sending 
    * nmas close session verb
   */
typedef int (NMAS_CALLBACK *pFunc_NMAS_AtEnd)(
   void       *arg,               
   int         loginResult,  /* login return code */
   MAF_Handle  mh ); 


NMASAPI int NMAS_ClientLoginEx(
   void                 *loginInfo,
   int                   loginInfoType,
   pFunc_NMAS_Transport  transFunc,       /* transport callback */       
   void                 *transArg,        /* passed to transFunc */      
   pFunc_NMAS_AtEnd      atEndFunc,       /* atEnd processing callback */
   void                 *atEndArg,        /* passed to atEndFunc */      
   int                   options);

NMASAPI int NMAS_ClientLoginExV2(
   void                 *loginInfo,
   int                   loginInfoType,
   pFunc_NMAS_Transport  transFunc,       /* transport callback */       
   void                 *transArg,        /* passed to transFunc */      
   pFunc_NMAS_AtEnd      atEndFunc,       /* atEnd processing callback */
   void                 *atEndArg,        /* passed to atEndFunc */      
   int                   options,
   MAF_Handle           *mafhandle);

typedef int (*pFunc_NMAS_ClientLoginEx)(
   void                 *loginInfo,
   int                   loginInfoType,
   pFunc_NMAS_Transport  transFunc,       
   void                 *transArg,        
   pFunc_NMAS_AtEnd      atEndFunc,       
   void                 *atEndArg,        
   int                   options);


typedef int (*pFunc_NMAS_ClientLoginExV2)(
   void                 *loginInfo,
   int                   loginInfoType,
   pFunc_NMAS_Transport  transFunc,       
   void                 *transArg,        
   pFunc_NMAS_AtEnd      atEndFunc,       
   void                 *atEndArg,        
   int                   options,
   MAF_Handle           *mafhandle);

NMASAPI int NMAS_ClientLoginExV2Init(
   MAF_Handle           *mhandle);

NMASAPI int NMAS_ClientLoginExV2Finish(
   MAF_Handle           *mhandle);



/*************************************************************************
   Backward compatibility with NMAS 2.x clients
*************************************************************************/
   
   /* Legacy - used by NMAS_ClientLogin - uses Novell specific "unicode" strings
   */
typedef struct _NMAS_ClientLoginRequestInfo
{
   unicode *userName;          
   unicode *reqSequence;       
   unicode *treeName;          
   unicode *reqClearance;      
   unicode *password;          
} NMAS_ClientLoginRequestInfo, *pNMAS_ClientLoginRequestInfo;


typedef pFunc_NMAS_Transport NMAS_ClientLoginTransportFcn;

   /* legacy routine - accepts "unicode" (2 byte) strings
   */
NMASAPI int NMAS_ClientLogin(
   NMAS_ClientLoginRequestInfo *rInfo,
   NMAS_ClientLoginTransportFcn transportFcn,
   void                        *transportArg, /* passed to callback */
   long                         features, 
   long                         options, 
   void                        *reserved1, 
   void                        *reserved2);

   /* function pointer typedef 
   */
typedef int (*pFunc_NMAS_ClientLogin)(
   NMAS_ClientLoginRequestInfo *rInfo,
   NMAS_ClientLoginTransportFcn transportFcn,
   void                        *transportArg,
   long                         features, 
   long                         options, 
   void                        *reserved1, 
   void                        *reserved2);

#endif
