/******************************************************************************

  Copyright (c) 1997-2008 Novell, Inc.  All Rights Reserved.                      

  THIS WORK IS  SUBJECT  TO  U.S.  AND  INTERNATIONAL  COPYRIGHT  LAWS  AND
  TREATIES.   NO  PART  OF  THIS  WORK MAY BE  USED,  PRACTICED,  PERFORMED
  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,  ABRIDGED, CONDENSED,
  EXPANDED,  COLLECTED,  COMPILED,  LINKED,  RECAST, TRANSFORMED OR ADAPTED
  WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION
  OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
  CRIMINAL AND CIVIL LIABILITY.

 *****************************************************************************/
 
#ifndef	_MAF_H
#define	_MAF_H

#include <nmasapi.h>
#include <unicode.h>
#include <methodcb.h>

#ifndef	MAF_Handle
#define	MAF_Handle	void *
#endif

#define GLOBAL_MID_LEN	4
#define GLOBAL_MID		0x00000000

#define NMAS_AID_USERNAME						1
#define NMAS_AID_TREENAME						2
#define NMAS_AID_DS_CONTEXT						3  // ths AID set by CIFS and AFP
#define NMAS_AID_REQUESTED_READ_CLEARANCE		4  // name is misleading -- replace with following name
#define NMAS_AID_REQUESTED_CLEARANCE			4  // more meaningful name
#define NMAS_AID_REQUESTED_WRITE_CLEARANCE		5
#define NMAS_AID_PASSWORD						6
#define NMAS_AID_PIN							7
#define NMAS_AID_NAME_SPACE						8
#define NMAS_AID_ROLE							9
#define NMAS_AID_CLEARANCE_FLAG					10
#define NMAS_AID_REQUESTED_METHODS				11  // name is misleading -- replace with following name
#define NMAS_AID_REQUESTED_LOGIN_SEQUENCE		11	// more meaningful name
#define NMAS_AID_MAF_CONTEXT					12
#define NMAS_AID_SERVER_MAF_HANDLE				13
#define NMAS_AID_CLIENT_MAF_HANDLE				14
#define NMAS_AID_SESSION_CONTEXT				15
#define NMAS_AID_CONNECTION						16
#define NMAS_AID_LDAP_CONTEXT					17
#define NMAS_AID_LM_CONFIG_DATA					18
#define NMAS_AID_LM_SECRET_DATA					19
#define NMAS_AID_LD_CONFIG_DATA					20
#define NMAS_AID_LD_SECRET_DATA					21
#define NMAS_AID_USER_CONFIG_DATA				22
#define NMAS_AID_USER_SECRET_DATA				23
#define NMAS_AID_USER_GLOBAL_CONFIG_DATA		24
#define NMAS_AID_USER_GLOBAL_SECRET_DATA		25
#define NMAS_AID_NETWORK_ADDRESS				26
#define NMAS_AID_LCM_CALLBACK					27
#define NMAS_AID_GA_INFO						28
#define NMAS_AID_DISCONNECTED_FLAG				29
#define NMAS_AID_LM_DN							30
#define NMAS_AID_LM_GLOBAL_CONFIG_DATA			31
#define NMAS_AID_LM_GLOBAL_SECRET_DATA			32
#define NMAS_AID_GET_PASSWORD_POLICY_DN			33
#define NMAS_AID_PROMPT_FOR_PASSWORD			34
#define NMAS_AID_OPTIONS                        35
#define NMAS_AID_FEATURES                       36
#define NMAS_AID_PWD_WARNING                    37
#define NMAS_AID_VERIFY_ONLY                    38
#define NMAS_AID_DAL_CONTEXT                    39
#define NMAS_AID_TYPED_USERNAME                 40
#define NMAS_AID_SEC_LAYER_CONTEXT              41
#define NMAS_AID_SPECIFIC_METHOD_DATA           42
#define NMAS_AID_AUTHORIZED_LOGIN_SEQUENCES     43
// MAF_LAST_PREDEFINED_ATTRIBUTE must be the value of the last predefined MAF attribute ID above
#define MAF_LAST_PREDEFINED_ATTRIBUTE NMAS_AID_AUTHORIZED_LOGIN_SEQUENCES

/*	Prototypes of MAF Module support routines	*/

NMASAPI	int  MAF_Begin(MAF_Handle mh);
NMASAPI	int  MAF_Read(MAF_Handle mh, size_t *rLenp, void *rBufp);

/* Provides safer way of reading message from queue */
/* Unlike MAF_Read, MAF_ReadV2 requires the length of the buffer to be specified in *rLenp */
NMASAPI	int  MAF_ReadV2(MAF_Handle mh, size_t *rLenp, void *rBufp);

NMASAPI	int  MAF_WriteRead(MAF_Handle mh, size_t wLen, void *wBufp, size_t *rLenp, void *rBufp);
NMASAPI	int  MAF_Write(MAF_Handle mh, size_t wLen, void *wBufp);
NMASAPI	int  MAF_End(MAF_Handle mh, int status, size_t len, void *seedDatap);

NMASAPI	int  MAF_XRead(MAF_Handle mh, size_t *rLenp, void *rBufp);
NMASAPI	int  MAF_XWriteRead(MAF_Handle mh, size_t wLen, void *wBufp, size_t *wLenp, void *rBufp);
NMASAPI	int  MAF_XWrite(MAF_Handle nh, size_t wLen, void *wBufp);

// MAF_Mem* functions should be used in place of MAF_Malloc/MAF_ReAlloc/MAF_Free
NMASAPI void *MAF_MemMalloc(MAF_Handle mh, size_t s);
NMASAPI void *MAF_MemRealloc(MAF_Handle mh, void *p, size_t s);
NMASAPI void MAF_MemFree(MAF_Handle mh, void *p);

NMASAPI	void 	*MAF_Malloc(size_t s);
NMASAPI	void 	*MAF_Realloc(void *p, size_t s);
NMASAPI	void	MAF_Free(void *p);

NMASAPI	int	MAF_GetAttribute(MAF_Handle mh, int aid, void *uniTag, size_t *aLenInBytesp, void *aValuep);
NMASAPI	int	MAF_PutAttribute(MAF_Handle mh, int aid, void *uniTag, size_t aLenInBytes, void *aValuep);

NMASAPI	int	MAF_GetMethodAttribute(MAF_Handle mh, void *uniTag, size_t *aLenInBytesp, void *aValuep);
NMASAPI	int	MAF_PutMethodAttribute(MAF_Handle mh, void *uniTag, size_t aLenInBytes, void *aValuep);

NMASAPI	int	MAF_RecallLCM(MAF_Handle mh, nuint32 methodNumber, nuint32 methodData);

NMASAPI int	MAF_SetIdentity(MAF_Handle mh, nint reserved, nint ndsContext);

// Flags that may be passed to the MAF set password function from a SPM agent -- also see spm.h
#define SPM_ADMIN_REQUEST 	0x1			// Indicates that the password change request is not from the user
#define SPM_IGNORE_POLICY	0x8			// Indicates that the password policy will not be checked on the input password
#define SPM_EXPIRE_NOW		0x10		// Indicates that the new password will be marked expired
#define SPM_MIGRATION       0x100       // Indicates that the password is being migrated from another password

// The following functions are implemented by NMAS Server 2.2 and later

// Flags that may be returned from the get password functions -- also see spm.h
#define SPM_EXPIRED                     0x1
#define SPM_GRACE_LOGIN_LIMIT	        0x2
#define SPM_SPWD_MIGRATED               0x4	// simple password was copied to password
#define SPM_PWD_RE_ENCRYPT              0x8
NMASAPI int	MAF_SetPassword(MAF_Handle mh, size_t passwordLen, unicode *password, nuint32 setFlags);
NMASAPI int	MAF_GetPassword(MAF_Handle mh, size_t *passwordLen, unicode *password, nuint32 *infoFlags, nuint32 *remainingLifetime, nuint32 *remainingGraceLogins);

NMASAPI int	MAF_GetNDSPasswordHash(MAF_Handle mh, nuint32 *pseudoID, size_t *pwdHashLen, nuint8 *pwdHash, nuint32 *pwdExpired, nuint32 *remainingGraceLogins);

NMASAPI int	MAF_PolicyCheck(MAF_Handle mh, size_t passwordLen, unicode *password);
NMASAPI int	MAF_AllowPasswordSet (MAF_Handle mh);

NMASAPI int	MAF_TraceEnabled(MAF_Handle mh);
NMASAPI int	MAF_Trace(MAF_Handle mh, char *message);
NMASAPI int  MAF_TraceOnError(MAF_Handle mh, int err, char *message);
NMASAPI int  MAF_TraceEx(MAF_Handle mh, const char *format, ...);

// The following functions are implemented by NMAS Server 2.3 and later
NMASAPI int	MAF_GetPasswordEx(MAF_Handle mh, nuint32 getFlags, size_t *passwordLen, unicode *password, nuint32 *infoFlags, nuint32 *remainingLifetime, nuint32 *remainingGraceLogins);

// The following functions are implemented by NMAS Server 3.0 and later

NMASAPI int MAF_LogEvent(
                    MAF_Handle mh,
                    nuint32 logLevel,
                    nuint32 status,
                    unsigned char *statusMsg,   // optional
                    nuint32 methodEventID,      // optional
                    char *eventMsgFmt, ...);    // optional

#ifdef __cplusplus

extern "C++" {

#include <memory>

struct IPassword;

int MAF_GetPasswordEx(MAF_Handle mh, nuint32 getFlags, std::unique_ptr<IPassword> &password, nuint32 *outInfoFlags, nuint32 *remainingLifetime, nuint32 *remainingGraceLogins);

}

#endif


// The following are the valid values for log Level parameter of MAF_LogEvent
#define MAF_LOG_LEVEL_EMERGENCY    1
#define MAF_LOG_LEVEL_ALERT        2
#define MAF_LOG_LEVEL_CRITICAL     3
#define MAF_LOG_LEVEL_ERROR        4
#define MAF_LOG_LEVEL_WARNING      5
#define MAF_LOG_LEVEL_NOTICE       6
#define MAF_LOG_LEVEL_INFO         7
#define MAF_LOG_LEVEL_DEBUG        8

NMASAPI int MAF_SetSecurityLayerMethod(MAF_Handle mh);

/* Session type */
#define NMAS_SESSION_LOGIN  1
#define NMAS_SESSION_VERIFY 2
#define NMAS_SESSION_AUTHN  3
#define NMAS_SESSION_AUTHN_CRED_REFRESH 4
#define NMAS_SESSION_AUTHN_CRED_UPGRADE 5

NMASAPI int MAF_GetSessionType(MAF_Handle mh, nuint32 *sessionType);

#endif
	 
