 
/******************************************************************************

  Copyright (c) 2006 Novell, Inc.  All Rights Reserved.                      

  THIS WORK IS  SUBJECT  TO  U.S.  AND  INTERNATIONAL  COPYRIGHT  LAWS  AND
  TREATIES.   NO  PART  OF  THIS  WORK MAY BE  USED,  PRACTICED,  PERFORMED
  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,  ABRIDGED, CONDENSED,
  EXPANDED,  COLLECTED,  COMPILED,  LINKED,  RECAST, TRANSFORMED OR ADAPTED
  WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION
  OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
  CRIMINAL AND CIVIL LIABILITY.

 *****************************************************************************/

#ifndef __MAFDS_H___
#define __MAFDS_H___

#include <maf.h>

#define MAFDS_CONTEXT void*
#define INVALID_MAFDS_CONTEXT ((MAFDS_CONTEXT)-1)

#define MAFDS_MAX_DN_CHARS 256

#define MAFDS_MODVALUE void*
#define MAFDS_VALUE void*

/* Value types for reading and writing attributes. */
#define MAFDS_SYN_DISTNAME     1
#define MAFDS_SYN_CE_STRING    2 /* Case-exact string */
#define MAFDS_SYN_CI_STRING    3 /* Case-ignore string */
#define MAFDS_SYN_BOOLEAN      4
#define MAFDS_SYN_INTEGER      5
#define MAFDS_SYN_OCTET_STRING 6 /* Buffer with binary data. */
#define MAFDS_SYN_TIME         7 /* Four-byte time value (time_t) */

/* Values for "operation" parameter in MAFDS_InsertModValue */
#define MAFDS_MODOP_ADD_VALUE    1    /* Add a value to an attribute. (Creates the attribute if it does not exist.) */
#define MAFDS_MODOP_MODIFY_VALUE 2    /* Change a value for a single-valued attribute, or add a value to a multi-valued attribute. */
#define MAFDS_MODOP_DEL_VALUE    3    /* Remove a value from an attribute */
#define MAFDS_MODOP_DEL_ATTR     4    /* Remove an attribute and all of its values. */

typedef struct MAFDS_ATTRIBUTE
{
    unicode *attr;
    int     syntax;
}MAFDS_ATTRIBUTE;

typedef struct MAFDS_VALUE_DATA
{
    /* The value */
    void         *data;
    size_t       size;
    
    /* The attribute from which this value was read. */
    unicode     *attr;
    int         syntax;

}MAFDS_VALUE_DATA;

NMASAPI int MAFDS_CreateContext(
    MAF_Handle mh,
    MAFDS_CONTEXT *outContext);

NMASAPI int MAFDS_FreeContext(
    MAFDS_CONTEXT context);

/* Reads the attribtues specified in attrs. Use MAFDS_GetValueData to retrieve the individual values. */
NMASAPI int MAFDS_ReadAttributeValues(
    MAFDS_CONTEXT context,
    unicode *dn,
    MAFDS_ATTRIBUTE *attrs, /* Array of MAFDS_ATTRIBUTE structs. Specifies which attributes to read. */
    nuint32 numAttrs,
    MAFDS_VALUE *values,
    int *numValuesRead); 

NMASAPI int MAFDS_ReadInheritedAttributeValues(
    MAFDS_CONTEXT context,
    unicode *objectDN,
    MAFDS_ATTRIBUTE *attr,
    MAFDS_VALUE *values,
    nuint32 *numValuesRead,
    unicode *objectRead,
    nuint32 objectReadSize); /* Size in bytes, not characters. */

NMASAPI int MAFDS_FreeValues(
    MAFDS_CONTEXT inContext,
    MAFDS_VALUE values);

NMASAPI int MAFDS_GetValueData(
    MAFDS_CONTEXT context,
    MAFDS_VALUE values,
    MAFDS_VALUE_DATA *valueData);

NMASAPI int MAFDS_FreeValueData(
    MAFDS_CONTEXT context,
    MAFDS_VALUE_DATA *valueData);

NMASAPI int MAFDS_ListContainerEntries(
    MAFDS_CONTEXT context,
    unicode *dn,
    unicode *objectClass, /* NULL to list all objects */
    unicode ***entries);

NMASAPI int MAFDS_FreeContainerEntries(
    MAFDS_CONTEXT *context,
    unicode **entries);

/* Modification of attributes on an object This call loads up all pending modifications 
   into a buffer pointed to by outModValue. To execute the modify operation, 
   call MAFDS_ModifyEntry. */
NMASAPI int MAFDS_InsertModValue(
    MAFDS_CONTEXT context,
    int operation,
    MAFDS_ATTRIBUTE *attr, /* Attribute to be modified. */
    void *value, /* Value of an attribute to be added. */
    size_t valueSize,
    MAFDS_MODVALUE *modValues);

NMASAPI int MAFDS_ModifyEntry(
    MAFDS_CONTEXT context,
    unicode *dn,
    MAFDS_MODVALUE modValues);

NMASAPI int MAFDS_FreeModValues(
    MAFDS_CONTEXT context,
    MAFDS_MODVALUE modValues);

NMASAPI int MAFDS_GetParentContainer(
    MAFDS_CONTEXT context,
    unicode *objectDN,
    unicode *parentDN,
    nuint32 parentDNSize); /* Size in bytes, not characters. */

NMASAPI int MAFDS_GetPartitionRootContainer(
    MAFDS_CONTEXT context,
    unicode *objectDN,
    unicode *partRootDN,
    nuint32 partRootDNSize); /* Size in bytes, not characters. */

#endif

